@@ -1,81 +1,48 @@
-# $Id: Cache.pm,v 1.9 2002/12/25 17:59:21 matts Exp $
+# $Id: Cache.pm,v 1.18 2001/01/19 13:50:52 matt Exp $
 
 package Apache::AxKit::Cache;
 use strict;
 
 use Apache;
-use Apache::Constants qw(OK DECLINED SERVER_ERROR);
 use Apache::AxKit::Exception;
 use Digest::MD5 ();
-use Compress::Zlib qw(gzopen);
-use Fcntl qw(:flock O_RDWR O_WRONLY O_CREAT O_RDONLY);
+use Compress::Zlib;
+use Fcntl qw(:DEFAULT);
 
 # use vars qw/$COUNT/;
 
 sub new {
     my $class = shift;
     my ($r, $xmlfile, @extras) = @_;
-    
-    my $gzip = 0;
-    if ($xmlfile =~ /\.gzip/) {
-        $gzip++;
-#        @extras = grep(!/gzip/, @extras);
-    }
-    
-    local $^W; # suppress "Use of uninitialized value" warnings
     my $key = Digest::MD5->new->add(
             join(':', 
-                $r->hostname,
+                $r->get_server_name,
                 $r->get_server_port,
                 $xmlfile,
                 @extras
             ))->hexdigest;
     
     AxKit::Debug(7, "Cache: key = $key");
-
-    my $primary = substr($key,0,2,'');
-    my $secondary = substr($key,0,2,'');
     
 #    warn "New for: $xmlfile:" . join(':', @extras). "\n";
     
     my $cachedir = $AxKit::Cfg->CacheDir();
     
     my $no_cache;
-
-    if ($AxKit::Cfg->NoCache()) {
-        $no_cache = 1;
+    
+    if (!-e $cachedir) {
+        if (!mkdir($cachedir, 0777)) {
+            warn "Can't create cache directory '$cachedir': $!\n";
+            $no_cache = 1;
+        }
     }
     
-    if (!$no_cache) {
-       if (!-e $cachedir) {
-           if (!mkdir($cachedir, 0777)) {
-               AxKit::Debug(2, "Can't create cache directory '$cachedir': $!");
-               $no_cache = 1;
-           }
-       }
-
-       if (!-e "$cachedir/$primary") {
-           if (!mkdir("$cachedir/$primary", 0777)) {
-               AxKit::Debug(1, "Can't create cache directory '$cachedir/$primary': $!");
-               $no_cache = 1;
-           }
-       }
-       
-       if (!-e "$cachedir/$primary/$secondary") {
-           if (!mkdir("$cachedir/$primary/$secondary", 0777)) {
-               AxKit::Debug(1, "Can't create cache directory '$cachedir/$primary/$secondary': $!");
-               $no_cache = 1;
-           }
-       }
-   }
-
     my $self = bless { 
         apache => $r,
         key => $key, 
         no_cache => $no_cache, 
         dir => $cachedir,
-        file => "$cachedir/$primary/$secondary/$key",
-        gzip => $gzip,
+        file => "$cachedir/$key",
 #        extras => \@extras,
         }, $class;
 
@@ -88,18 +55,6 @@ sub new {
     return $self;
 }
 
-sub _get_stats {
-    my $self = shift;
-    return if $self->{mtime};
-    my @stats = stat($self->{file});
-    my $exists = -e _ && -r _;
-    if ($exists and $self->{gzip}) {
-        $exists = -e $self->{file} . '.gz' and -r _;
-    }
-    $self->{file_exists} = $exists;
-    $self->{mtime} = $stats[9];
-}
-
 # sub DESTROY {
 #     AxKit::Debug(7, "Cache->DESTROY Count: ".--$COUNT);
 # }
@@ -107,27 +62,29 @@ sub _get_stats {
 sub write {
     my $self = shift;
     return if $self->{no_cache};
-    AxKit::Debug(7, "[Cache] writing cache file $self->{file}");
     my $fh = Apache->gensym();
-    my $tmp_filename = $self->{file}."new$$";
-    if (sysopen($fh, $tmp_filename, O_WRONLY|O_CREAT)) {
-        # flock($fh, LOCK_EX);
-        # seek($fh, 0, 0);
-        # truncate($fh, 0);
+    if (sysopen($fh, $self->{file}.'new', O_RDWR|O_CREAT)) {
+        flock($fh, 2);
+        seek($fh, 0, 0);
+        truncate($fh, 0);
         print $fh $_[0];
         close $fh;
-        rename($tmp_filename, $self->{file}) 
+        rename($self->{file}.'new', $self->{file}) 
                 || throw Apache::AxKit::Exception::IO( -text => "Couldn't rename cachefile: $!");
     }
     else {
         throw Apache::AxKit::Exception::IO( -text => "Couldn't open cachefile for writing: $!");
     }
     
-    if ($self->{gzip} && $AxKit::Cfg->GzipOutput) {
-        AxKit::Debug(3, "Creating gzip output cache: $self->{file}.gz");
-        if (my $gz = gzopen($self->{file}.'new.gz', "wb")) {
-            $gz->gzwrite($_[0]);
-            $gz->gzclose();
+    if ($AxKit::Cfg->GzipOutput) {
+        AxKit::Debug(3, 'Creating gzip output cache');
+        my $fh = Apache->gensym();
+        if (sysopen($fh, $self->{file}.'new.gz', O_RDWR|O_CREAT)) {
+            flock($fh, 2);
+            seek($fh, 0, 0);
+            truncate($fh, 0);
+            print $fh ''.Compress::Zlib::memGzip($_[0]);
+            close $fh;
             rename($self->{file}.'new.gz', $self->{file}.'.gz')
                     || throw Apache::AxKit::Exception::IO( -text => "Couldn't rename gzipped cachefile: $!");
         }
@@ -142,11 +99,10 @@ sub read {
     return if $self->{no_cache};
     my $fh = Apache->gensym();
     if (sysopen($fh, $self->{file}, O_RDONLY)) {
-        flock($fh, LOCK_SH);
+        flock($fh, 1);
         local $/;
         return <$fh>;
-        # close($fh);
-        # close unlocks automatically
+        # closes and unlocks automatically upon loss of scope
     }
     return '';
 }
@@ -156,7 +112,7 @@ sub get_fh {
     return if $self->{no_cache};
     my $fh = Apache->gensym();
     if (sysopen($fh, $self->{file}, O_RDONLY)) {
-        flock($fh, LOCK_SH);
+        flock($fh, 1);
         return $fh;
     }
     else {
@@ -164,68 +120,27 @@ sub get_fh {
     }
 }
 
-sub set_type {
-    my $self = shift;
-    return if $self->{no_cache};
-    
-    my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}.'newtype', O_RDWR|O_CREAT)) {
-        flock($fh, LOCK_EX);
-        seek($fh, 0, 0);
-        truncate($fh, 0);
-        print $fh $_[0];
-        close $fh;
-        rename($self->{file}.'newtype', $self->{file}.'.type') 
-                || throw Apache::AxKit::Exception::IO( -text => "Couldn't rename type cachefile: $!");
-    }
-    else {
-        throw Apache::AxKit::Exception::IO( -text => "Couldn't open type cachefile for writing: $!");
-    }
-}
-
-sub get_type {
-    my $self = shift;
-    return if $self->{no_cache};
-    my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}.'.type', O_RDONLY)) {
-        flock($fh, LOCK_SH);
-        local $/;
-        return <$fh>;
-        # close($fh);
-        # close unlocks automatically
-    }
-    return '';
-}
-
 sub deliver {
     my $self = shift;
-    return SERVER_ERROR if $self->{no_cache};
+    return if $self->{no_cache};
     my $r = $self->{apache};
 
     {
         # get content-type
         AxKit::Debug(4, "Cache: Getting content-type");
-        if (my $type = $self->get_type) {
-            AxKit::Debug(4, "Cache: setting content-type: $type");
+        my $typecache = Apache::AxKit::Cache->new($r, $self->{key} . '.type');
+        if (my $type = $typecache->read()) {
             $r->content_type($type);
         }
     }
-
-    if ($r->content_type eq 'changeme' && !$r->notes('axkit_passthru_type')) {
-        $AxKit::Cfg->AllowOutputCharset(1);
-        $r->content_type('text/html; charset=' . ($AxKit::Cfg->OutputCharset || "UTF-8"));
-    }
-    elsif ($r->notes('axkit_passthru_type')) {
-        $r->content_type($AxKit::OrigType);
-    }
-
-
+    
     my ($transformer, $doit) = AxKit::get_output_transformer();
-
+    
     if ($doit) {
         AxKit::Debug(4, "Cache: Transforming content and printing to browser");
-        $r->pnotes('xml_string',$self->read());
-        return OK; # upstream deliver_to_browser should handle the rest
+        $r->send_http_header() unless lc($r->dir_config('Filter')) eq 'on';
+        $r->print( $transformer->( $self->read() ) );
+        throw Apache::AxKit::Exception::OK();
     }
     else {
         AxKit::Debug(4, "Cache: Sending untransformed content to browser");
@@ -233,18 +148,20 @@ sub deliver {
         # Make sure we unset PATH_INFO or wierd things can happen!
         $ENV{PATH_INFO} = '';
         $r->path_info('');
-
-        if ($self->{gzip} && $AxKit::Cfg->DoGzip) {
+        
+        if ($AxKit::Cfg->DoGzip) {
             AxKit::Debug(4, 'Cache: Delivering gzipped output');
             $r->filename($self->{file}.'.gz');
         }
         else {
             $r->filename($self->{file});
         }
-
-        return DECLINED;
+        
+        throw Apache::AxKit::Exception::Declined(
+                reason => "delivering cached copy"
+                );
     }
-
+    
 }
 
 sub reset {
@@ -254,21 +171,13 @@ sub reset {
 
 sub mtime {
     my $self = shift;
-    $self->_get_stats;
     return $self->{mtime} if exists $self->{mtime};
-    return ($self->{mtime} = (stat($self->{file}))[9]);
-}
-
-sub has_changed {
-    my $self = shift;
-    my $time = shift;
-    return $self->mtime > $time;
+    return ($self->{mtime} = -M $self->{file});
 }
 
 sub exists {
     my $self = shift;
     return if $self->{no_cache};
-    $self->_get_stats;
     return $self->{file_exists} if exists $self->{file_exists};
     return ($self->{file_exists} = -e $self->{file});
 }
@@ -286,6 +195,17 @@ sub no_cache {
     if ($_[0]) {
         AxKit::Debug(8, "Turning off cache!");
         $self->{no_cache} = 1;
+#         my $r = $self->{apache};
+#         my $fh = Apache->gensym();
+#         if (sysopen($fh, $self->{file}, O_RDONLY)) {
+#             flock($fh, 1);
+#             $r->send_http_header();
+#             while (<$fh>) {
+#                 $r->print($_);
+#             }
+#             close $fh;
+#         }
+        
         $self->reset();
     }
     
