@@ -1,51 +1,26 @@
-# $Id: Provider.pm,v 1.12 2002/07/05 19:02:36 matts Exp $
+# $Id: Provider.pm,v 1.18 2001/01/15 16:56:27 matt Exp $
 
 package Apache::AxKit::Provider;
 use strict;
 
-use Apache::AxKit::Exception;
-use Apache::Constants qw(OK DECLINED);
-#use XML::Parser;
+use Apache::AxKit::Exception ':try';
 
 # use vars qw/$COUNT/;
 
-sub new_style_provider {
-    my $class = shift;
-    my $apache = shift;
-    my $self = bless { apache => $apache }, $class;
-
-    if (my $alternate = $AxKit::Cfg->StyleProviderClass()) {
-        AxKit::Debug(8, "Style Provider Override: $alternate" );
-        AxKit::reconsecrate($self, $alternate);
-    }
-
-    $self->init(@_);
-
-    AxKit::add_depends($self->key());
-
-    return $self;
-}
-
-sub new_content_provider {
-    my $class = shift;
-    my $apache = shift;
-    my $self = bless { apache => $apache }, $class;
-
-    if (my $alternate = $AxKit::Cfg->ContentProviderClass()) {
-        AxKit::Debug(8, "Content Provider Override: $alternate" );
-        AxKit::reconsecrate($self, $alternate);
-    }
-
-    $self->init(@_);
-
-    AxKit::add_depends($self->key());
-
-    return $self;
-}
-
 sub new {
     my $class = shift;
-    return $class->new_content_provider( @_ );
+    my $apache = shift;
+    my $self = bless { apache => $apache }, $class;
+    
+    if (my $alternate = $AxKit::Cfg->ProviderClass()) {
+        AxKit::reconsecrate($self, $alternate);
+    }
+    
+    $self->init(@_);
+    
+#     AxKit::Debug(7, "Provider->new Count: " . ++$COUNT);
+    
+    return $self;
 }
 
 sub init {
@@ -61,26 +36,10 @@ sub apache_request {
     return $self->{apache};
 }
 
-sub has_changed {
-    my $self = shift;
-    my $time = shift;
-    return 1 unless defined $time;
-    return $self->mtime > $time;
-}
-
-sub decline {
-    my $self = shift;
-
-    AxKit::Debug(4, "provider declined");
-    return DECLINED;
-}
-
 sub get_ext_ent_handler {
     my $self = shift;
     return sub {
         my ($e, $base, $sysid, $pubid) = @_;
-#        warn "ext_ent: base => $base, sys => $sysid, pub => $pubid\n";
-        AxKit::Debug(6, "Provider get_ext_ent_handler for $sysid");
         if ($sysid =~ /^http:/) {
             if ($pubid) {
                 return ''; # do not bring in public DTD's
@@ -89,9 +48,8 @@ sub get_ext_ent_handler {
                 require HTTP::GHTTP;
             };
             if ($@) {
-                require LWP::Simple;
-                import LWP::Simple;
-                return get($sysid) || die "Cannot get $sysid";
+                require LWP;
+                return XML::Parser::lwp_ext_ent_handler(@_);
             }
             my $r = HTTP::GHTTP->new($sysid);
             $r->process_request;
@@ -103,23 +61,13 @@ sub get_ext_ent_handler {
             }
             die "Cannot download https (SSL) or ftp URL's yet. Patches welcome";
         }
-
-        # create a subrequest, so we get the right AxKit::Cfg for the URI
-        my $apache = AxKit::Apache->request;
-        my $sub = $apache->lookup_uri(AxKit::FromUTF8($sysid));
-        local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
-    
+        
 #        warn "File provider ext_ent_handler called with '$sysid'\n";
-        my $provider = Apache::AxKit::Provider->new($sub);
-        
-#        warn "Got provider with key: ", $provider->key, "\n";
+        my $provider = Apache::AxKit::Provider->new(
+                Apache->request,
+                uri => $sysid
+                );
         my $str = $provider->get_strref;
-#        warn "Returning string with length: ", length($$str), "\n";
-
-        undef $provider;
-        undef $apache;
-        undef $sub;
-        
         return $$str;
     };
 }
@@ -127,82 +75,80 @@ sub get_ext_ent_handler {
 sub get_styles {
     my $self = shift;
     my ($media, $pref_style) = @_;
-
+    
     if ($pref_style eq '#default') {
         undef $pref_style;
     }
-
-    my $xml_styles = [];
+    
+    my $styles = [];
+    my $ext_ents = [];
     my $vals = [];
-
+    
     my $key = $self->key();
-
+    
     # need to extract the following from the XML file:
     #   DocType Public Identifier
     #   DTD filename
     #   Root element name (including namespace)
     # use three element array @$vals
-
-    if (defined &Apache::AxKit::Provider::xs_get_styles_fh) {
-        AxKit::Debug(2, "using XS get_styles (libxml2)");
-        my ($xs_styles, $doctype, $dtd, $root) =
-                $self->xs_get_styles($media, $pref_style);
-        @$xml_styles = @$xs_styles unless $AxKit::Cfg->IgnoreStylePI();
-        @$vals = ($doctype, $dtd, $root);
-    }
-    else {
-        require XML::Parser;
-
-        AxKit::Debug(4, "get_styles: creating XML::Parser");
-
-        my $handlers = {
-                    Start => \&parse_start,
-                    Doctype => \&parse_dtd,
-                    $AxKit::Cfg->IgnoreStylePI() ? () : (Proc => \&parse_pi),
-                    };
-
-        my $xml_parser = XML::Parser->new(
-                Namespaces => 1,
+    
+    AxKit::Debug(4, "get_styles: creating XML::Parser");
+    
+    my $xml_parser = XML::Parser->new(
+                ParseParamEnt => 1, 
                 ErrorContext => 2,
-                Handlers => $handlers,
-            );
+                Namespaces => 1,
+                );
+    $xml_parser->setHandlers(
+                Start => \&parse_start,
+                Doctype => \&parse_dtd,
+                Proc => \&parse_pi,
+                Entity => \&parse_entity_decl,
+                );
 
-        my $to_parse;
-        eval {
-            $to_parse = $self->get_fh();
-        };
-        if ($@) {
-            $to_parse = ${ $self->get_strref(); };
-        }
-        
+    if (my $entity_handler = $self->get_ext_ent_handler()) {
+        $xml_parser->setHandlers(
+				 ExternEnt => $entity_handler,
+				);
+    }
+    
+    my $to_parse = try { 
+        $self->get_fh();
+    } catch Error with {
+        ${ $self->get_strref(); };
+    };
+    
+    try {
         AxKit::Debug(4, "get_styles: calling XML::Parser->parse('$key')");
         $xml_parser->parse(
                 $to_parse,
                 XMLStyle_preferred => $pref_style,
-                XMLStyle_media => $media,
-                XMLStyle_style => $xml_styles,
+                XMLStyle_style => $styles,
+                XMLStyle_ext_ents => $ext_ents,
                 XMLStyle_vals => $vals,
                 XMLStyle_style_screen => [],
+                XMLStyle_media => $media,
                 );
-            
-        AxKit::Debug(4, "get_styles: parse returned successfully");
     }
-    
-    foreach my $style (@$xml_styles) {
-        $style->{title} ||= '#default';
-    }
-    
+    catch Error with {
+        my $E = shift;
+        AxKit::Debug(4, "get_styles: parse returned: >> $E <<");
+        if ("$E" !~ /^OK/) { # note that regex bindings don't stringify in 5.6.0 - fixed in 5.6.1
+            throw Apache::AxKit::Exception::Error(
+                    -text => "Parsing '$key' returned: $E\n"
+                    );
+        }
+    };
+        
+    AxKit::Debug(4, "get_styles: parse returned successfully");
+
     # Let GetMatchingProcessors to process the @$styles array
-    {
-      local $^W; # suppress "Use of uninitialized value" warnings
-      AxKit::Debug(4, "Calling GetMatchingProcessors with ($media, $pref_style, $vals->[0], $vals->[1], $vals->[2])");
-    }
     my @styles = $AxKit::Cfg->GetMatchingProcessors($media,
-		$pref_style, @$vals[0 .. 2], $xml_styles, $self);
+		$pref_style, @$vals[0 .. 2], $styles);
     
     if (!@styles) {
         throw Apache::AxKit::Exception::Declined(
-                reason => "No styles defined for '$key'"
+                -text => "No styles defined for '$key'"
                 );
     }
     
@@ -222,77 +168,18 @@ sub get_styles {
         $style->{module} = $mapto;
 
         # first load module if it's not already loaded.
-        eval {
+        try {
             AxKit::load_module($mapto);
-        };
-        if ($@) {
+        }
+        catch Error with {
+            my $E = shift;
             throw Apache::AxKit::Exception::Error(
-                    -text => "Load of '$mapto' failed with: $@"
+                    reason => "Load of '$mapto' failed with: $E"
                     );
-        }
-    }
-    
-    return \@styles;
-}
-
-sub xs_get_styles {
-    my $self = shift;
-    my ($media, $pref_style) = @_;
-    
-    my $bits;
-    eval {
-        my $fh = $self->get_fh();
-        AxKit::Debug(4, "calling xs_get_styles_fh()");
-        $bits = xs_get_styles_fh($self->apache_request, $fh);
-    };
-    if ($@) {
-        my $strref = $self->get_strref();
-        AxKit::Debug(4, "calling xs_get_styles_str()");
-        $bits = xs_get_styles_str($self->apache_request, $$strref);
-    }
-    
-    my @xml_stylesheet = @{$bits->[0]};
-    my %attribs = @{$bits->[2]};
-    my $element = $bits->[1];
-    
-    # resolve namespaces (libxml doesn't!)
-    if ($attribs{'xmlns'} && $element !~ /:/) {
-        $element = "{$attribs{xmlns}}$element";
-    }
-    elsif ($element =~ /^(.*):(.*)$/) {
-        my ($prefix, $el) = ($1, $2);
-        my $ns = $attribs{"xmlns:$prefix"};
-        $element = "{$ns}$el";
-    }
-    
-    my $e = {
-        XMLStyle_style => [], 
-        XMLStyle_style_screen => [],
-        XMLStyle_preferred => $pref_style,
-        XMLStyle_media => $media,
         };
-    
-    foreach my $pi (@xml_stylesheet) {
-        parse_pi($e, "xml-stylesheet", $pi);
     }
     
-    if (!@{$e->{XMLStyle_style}} && !$e->{XMLStyle_style_persistant}) {
-        if ($e->{XMLStyle_style_screen_persistant}) {
-            push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen_persistant}};
-        }
-        if (@{$e->{XMLStyle_style_screen}}) {
-            push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen}};
-        }
-    }
-    elsif ($e->{XMLStyle_style_persistant}) {
-        unshift @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_persistant}};
-    }
-    
-    { local $^W;
-      AxKit::Debug(4, "xs_get_styles returned: $bits->[3], $bits->[4], $element");
-    }
-    
-    return ($e->{XMLStyle_style}, $bits->[3], $bits->[4], $element);
+    return \@styles, $ext_ents;
 }
 
 sub parse_start {
@@ -301,11 +188,14 @@ sub parse_start {
     # use James Clark's universal name format
     $e->{XMLStyle_vals}[2] = $ns ? "{$ns}$el" : $el;
     
+#    warn "styles: ", scalar @{$e->{XMLStyle_style_screen_persistant}}, "\n";
+    
     if (!@{$e->{XMLStyle_style}} && !$e->{XMLStyle_style_persistant}) {
         if ($e->{XMLStyle_style_screen_persistant}) {
             push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen_persistant}};
         }
         if (@{$e->{XMLStyle_style_screen}}) {
+    #        warn "Matching style for media ", $e->{XMLStyle_media}, " not found. Using screen media stylesheets instead\n";
             push @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_screen}};
         }
     }
@@ -313,7 +203,17 @@ sub parse_start {
         unshift @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_persistant}};
     }
     
-    $e->finish();
+    die "OK\n";
+}
+
+sub parse_entity_decl {
+    my $e = shift;
+    my ($name, $val, $sysid, $pubid, $ndata) = @_;
+#    warn "external entity: '$sysid'\n";
+    if (!defined $val) {
+        # external entity - save so the cache gets done properly!
+        push @{$e->{XMLStyle_ext_ents}}, $sysid;
+    }
 }
 
 sub parse_dtd {
@@ -450,27 +350,18 @@ Apache::AxKit::Provider - base Provider class
 
 =head1 SYNOPSIS
 
-Override the base ContentProvider class and enable it using:
+Override the base Provider class and enable it using:
 
-    AxContentProvider MyClass
+    AxProvider MyClass
     
     # alternatively use:
-    # PerlSetVar AxContentProvider MyClass
-
-Override the base StyleProvider class and enable it using:
-
-    AxStyleProvider MyClass
-    
-    # alternatively use:
-    # PerlSetVar AxStyleProvider MyClass
+    # PerlSetVar AxProvider MyClass
 
 =head1 DESCRIPTION
 
-The Provider class is used to read in the data sources for the given URL.
-The ContentProvider handles the task of returning the data for the XML source
-document, while the StyleProvider fetches the data for any stylesheets that
-will be applied to that source document. The default for each is Provider::File, 
-which reads from the filesystem, although obviously you can read from just about anywhere.
+The Provider class is used to read in the data source for the given URL.
+The default Provider is Provider::File, which reads from the filesystem,
+although obviously you can read from just about anywhere.
 
 Should you wish to override the default Provider, these are the methods
 you need to implement:
@@ -479,7 +370,7 @@ you need to implement:
 
 Determine whether or not to process this URL. For example, you don't want
 to process a directory request, or if the resource doesn't exist. Return 1
-to tell AxKit to process this URL, or die with a Declined exception (with
+to tell AxKit to process this URL, or die with a Declined exception (with 
 a reason) if you do not wish to process this URL.
 
 =head2 mtime()
@@ -488,21 +379,11 @@ Return the last modification time in days before the current time.
 
 =head2 get_styles()
 
-Extract the stylesheets from the XML resource. Should return an array
-ref of styles. The style entries are hashes refs with required keys
-'href', 'module' and 'type'.
-
-A sample return value of get_styles() could be:
-
-  [{ type => "text/xsl",
-     href => "path/to/your/stylesheet.xsl",
-     module => "Apache::AxKit::Language::LibXSLT" }]
-
-This allows one to use a different stylemapping than the default
-server configuration. If 'module' is ommited AxKit will apply the 
-given default module for the type as it was added by 'AxAddStyleMap'
-in the configuration.
-
+Extract the stylesheets and external entities from the XML resource. Should
+return a list of ($styles, $ext_ents). Both are array refs, the style
+entries are hashes refs with required keys 'href' and 'type'. The external
+entities entries are scalars containing the system identifier of the 
+external entity.
 
 =head2 get_fh()
 
